@@ -208,15 +208,19 @@ local _M = {
     "jwe-decrypt",
     "key-auth",
     "consumer-restriction",
+    "attach-consumer-label",
     "forward-auth",
     "opa",
     "authz-keycloak",
     "proxy-cache",
     "body-transformer",
+    "ai-prompt-template",
+    "ai-prompt-decorator",
     "proxy-mirror",
     "proxy-rewrite",
     "workflow",
     "api-breaker",
+    "ai-proxy",
     "limit-conn",
     "limit-count",
     "limit-req",
