@@ -538,7 +538,7 @@ local function load_full_data(self, dir_res, headers)
             end
 
             if data_valid and self.checker then
-                data_valid, err = self.checker(item.value)
+                data_valid, err = self.checker(item.value, item.key)
                 if not data_valid then
                     log.error("failed to check item data of [", self.key,
                               "] err:", err, " ,val: ", json.delay_encode(item.value))
@@ -674,7 +674,7 @@ local function sync_data(self)
         end
 
         if data_valid and res.value and self.checker then
-            data_valid, err = self.checker(res.value)
+            data_valid, err = self.checker(res.value, res.key)
             if not data_valid then
                 log.error("failed to check item data of [", self.key,
                           "] err:", err, " ,val: ", json.delay_encode(res.value))
