@@ -49,7 +49,6 @@ local ngx             = ngx
 local get_method      = ngx.req.get_method
 local ngx_exit        = ngx.exit
 local math            = math
-local error           = error
 local ipairs          = ipairs
 local ngx_now         = ngx.now
 local ngx_var         = ngx.var
@@ -114,18 +113,8 @@ function _M.http_init_worker()
     -- for testing only
     core.log.info("random test in [1, 10000]: ", math.random(1, 10000))
 
-    -- Because go's scheduler doesn't work after fork, we have to load the gRPC module
-    -- in each worker.
-    core.grpc = require("apisix.core.grpc")
-    if type(core.grpc) ~= "table" then
-        core.grpc = nil
-    end
+    require("apisix.events").init_worker()
 
-    local we = require("resty.worker.events")
-    local ok, err = we.configure({shm = "worker-events", interval = 0.1})
-    if not ok then
-        error("failed to init worker event: " .. err)
-    end
     local discovery = require("apisix.discovery.init").discovery
     if discovery and discovery.init_worker then
         discovery.init_worker()
@@ -159,6 +148,7 @@ function _M.http_init_worker()
     apisix_upstream.init_worker()
     require("apisix.plugins.ext-plugin.init").init_worker()
 
+    control_api_router.init_worker()
     local_conf = core.config.local_conf()
 
     if local_conf.apisix and local_conf.apisix.enable_server_tokens == false then
@@ -195,6 +185,7 @@ function _M.http_ssl_client_hello_phase()
         core.log.error("failed to find SNI: " .. (err or advise))
         ngx_exit(-1)
     end
+    local tls_ext_status_req = apisix_ssl.get_status_request_ext()
 
     local ngx_ctx = ngx.ctx
     local api_ctx = core.tablepool.fetch("api_ctx", 0, 32)
@@ -205,6 +196,7 @@ function _M.http_ssl_client_hello_phase()
     ngx_ctx.matched_ssl = api_ctx.matched_ssl
     core.tablepool.release("api_ctx", api_ctx)
     ngx_ctx.api_ctx = nil
+    ngx_ctx.tls_ext_status_req = tls_ext_status_req
 
     if not ok then
         if err then
@@ -313,7 +305,7 @@ end
 
 local function verify_tls_client(ctx)
     if apisix_base_flags.client_cert_verified_in_handshake then
-        -- For apisix-base, there is no need to rematch SSL rules as the invalid
+        -- For apisix-runtime, there is no need to rematch SSL rules as the invalid
         -- connections are already rejected in the handshake
         return true
     end
@@ -577,6 +569,11 @@ end
 
 
 function _M.http_access_phase()
+    -- from HTTP/3 to HTTP/1.1 we need to convert :authority pesudo-header
+    -- to Host header, so we set upstream_host variable here.
+    if ngx.req.http_version() == 3 then
+        ngx.var.upstream_host = ngx.var.host .. ":" .. ngx.var.server_port
+    end
     local ngx_ctx = ngx.ctx
 
     -- always fetch table from the table pool, we don't need a reused api_ctx
@@ -1022,11 +1019,8 @@ function _M.stream_init_worker()
     require("apisix.http.service").init_worker()
     apisix_upstream.init_worker()
 
-    local we = require("resty.worker.events")
-    local ok, err = we.configure({shm = "worker-events-stream", interval = 0.1})
-    if not ok then
-        error("failed to init worker event: " .. err)
-    end
+    require("apisix.events").init_worker()
+
     local discovery = require("apisix.discovery.init").discovery
     if discovery and discovery.init_worker then
         discovery.init_worker()
